/* crypto/request.c */

#include <openssl/bio.h>
#include <openssl/err.h>
#include <openssl/rsa.h>
#include <openssl/evp.h>
#include <openssl/x509.h>
#include <openssl/x509v3.h>
#include <openssl/objects.h>
#include <openssl/pem.h>
#include <openssl/asn1.h>
#include <openssl/txt_db.h>

#include <ocsp_data.h>
#include <ocsp_req_st.h>

#undef BSIZE
#define BSIZE 256

/* Direct ASN1 functions */
int i2d_OCSP_REQ ( OCSP_REQUEST *a, unsigned char **pp) {
	/*
	** ASN1 functions (hope this works fine ... please give feedback,
	** I am not at ease with this OpenSSL functions ... :-D
	*/

	int num=0;

	M_ASN1_I2D_vars(a);

        M_ASN1_I2D_len(a->tbsRequest, i2d_OCSP_REQINFO);
        M_ASN1_I2D_len_EXP_opt(a->optionalSignature, i2d_OCSP_SIGNATURE,0,num);
        M_ASN1_I2D_seq_total();
        M_ASN1_I2D_put(a->tbsRequest, i2d_OCSP_REQINFO);
        M_ASN1_I2D_put_EXP_opt(a->optionalSignature, i2d_OCSP_SIGNATURE,0,num);
        M_ASN1_I2D_finish();

}

int i2d_OCSP_REQINFO(OCSP_REQINFO *a, unsigned char **pp)
        {
        int v1=0,v2=0,v3=0;
        M_ASN1_I2D_vars(a);

        M_ASN1_I2D_len_EXP_opt(a->version,i2d_ASN1_INTEGER,0,v1);
        M_ASN1_I2D_len_EXP_opt(a->requestorName,i2d_GENERAL_NAME,1,v2);
        M_ASN1_I2D_len_SEQUENCE(a->requestList,i2d_OCSP_ONEREQ);

/*
        M_ASN1_I2D_len_EXP_SET_opt(a->requestExtensions,i2d_X509_EXTENSION,2,V_ASN1_SEQUENCE,v3);
*/
        M_ASN1_I2D_seq_total();

        M_ASN1_I2D_put_EXP_opt(a->version,i2d_ASN1_INTEGER,0,v1);
        M_ASN1_I2D_put_EXP_opt(a->requestorName,i2d_GENERAL_NAME,1,v2);
        M_ASN1_I2D_put_SEQUENCE(a->requestList,i2d_OCSP_ONEREQ);

/*
        M_ASN1_I2D_put_EXP_SET_opt(a->requestExtensions,i2d_X509_EXTENSION,2,V_ASN1_SEQUENCE,v3);
*/
        M_ASN1_I2D_finish();
        }


int i2d_OCSP_CERTID( OCSP_CERTID *a, unsigned char **pp)
{
        M_ASN1_I2D_vars(a);

        M_ASN1_I2D_len(a->hashAlgorithm, i2d_X509_ALGOR);
        M_ASN1_I2D_len(a->issuerNameHash, i2d_ASN1_OCTET_STRING);
        M_ASN1_I2D_len(a->issuerKeyHash, i2d_ASN1_OCTET_STRING);
        M_ASN1_I2D_len(a->serialNumber, i2d_ASN1_INTEGER);
        M_ASN1_I2D_seq_total();
        M_ASN1_I2D_put(a->hashAlgorithm, i2d_X509_ALGOR);
        M_ASN1_I2D_put(a->issuerNameHash, i2d_ASN1_OCTET_STRING);
        M_ASN1_I2D_put(a->issuerKeyHash, i2d_ASN1_OCTET_STRING);
        M_ASN1_I2D_put(a->serialNumber, i2d_ASN1_INTEGER);
        M_ASN1_I2D_finish();
}

int i2d_OCSP_SIGNATURE(OCSP_SIGNATURE *a,unsigned char **pp) {
        int v=0;

        M_ASN1_I2D_vars(a);

        M_ASN1_I2D_len(a->signatureAlgorithm, i2d_X509_ALGOR);
        M_ASN1_I2D_len(a->signature, i2d_ASN1_BIT_STRING);
        M_ASN1_I2D_len_EXP_SET_opt(a->certs,i2d_X509,0,V_ASN1_SEQUENCE,v);
        M_ASN1_I2D_seq_total();
        M_ASN1_I2D_put(a->signatureAlgorithm, i2d_X509_ALGOR);
        M_ASN1_I2D_put(a->signature, i2d_ASN1_BIT_STRING);
        M_ASN1_I2D_put_EXP_SET_opt(a->certs,i2d_X509,0,V_ASN1_SEQUENCE,v);
        M_ASN1_I2D_finish();
        }

int i2d_OCSP_ONEREQ(OCSP_ONEREQ *a, unsigned char **pp)
        {
        int v=0;
        M_ASN1_I2D_vars(a);

        M_ASN1_I2D_len(a->reqCert, i2d_OCSP_CERTID);
/*
        M_ASN1_I2D_len_EXP_SET_opt(a->singleRequestExtensions,
                                   i2d_X509_EXTENSION,0,V_ASN1_SEQUENCE,v);
*/
        M_ASN1_I2D_seq_total();
        M_ASN1_I2D_put(a->reqCert, i2d_OCSP_CERTID);
/*
        M_ASN1_I2D_put_EXP_SET_opt(a->singleRequestExtensions,
                                   i2d_X509_EXTENSION,0,V_ASN1_SEQUENCE,v);
*/
        M_ASN1_I2D_finish();
        }

